﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Spread.Common.Features.Utilities;

namespace Spread.ControlExplorer.Components
{
    public delegate void StackNodeEventHandler(object sender, StackNodeEventArgs e);
    public partial class StackNodeControl : UserControl
    {
        #region events
        public event StackNodeEventHandler StackNodeClick;
        #endregion

        #region fields

        private Color _selectionColor = Color.FromArgb(241, 241, 241);
        private Image _collapsedImage;
        private StackedNodeCollection _nodes;
        private int _level;
        private bool isExpanded, showImage = true;

        #endregion

        public Color SelectionColor
        {
            get { return _selectionColor; }
            set
            {
                _selectionColor = value;
                lblText.BackColor = mainPanel.BackColor = pnlPB.BackColor = pbMain.BackColor = value;
            }
        }
        public Color HoverBackColor { get; set; }
        public Color HoverForeColor { get; set; }
        public Image ExpandedImage { get; set; }
        public Image CollapsedImage
        {
            get
            { return this._collapsedImage; }
            set
            {
                _collapsedImage = value;
                if (!this.isExpanded)
                    pbMain.Image = value;
            }
        }
        bool _ShouldBeSeen = true;
        public bool ShouldBeSeen
        {
            get
            {
                return _ShouldBeSeen;
            }
            set
            {
                if (_ShouldBeSeen != value)
                {
                    _ShouldBeSeen = value;
                }
                OnNodeVisibleChange();
            }
        }

        bool _IsVisible = false;
        public bool IsVisible
        {
            get
            {
                return _IsVisible;
            }
            set
            {
                if (_IsVisible != value)
                {
                    _IsVisible = value;
                }
                OnNodeVisibleChange();
            }
        }

        public bool ShowImage
        {
            get { return showImage; }
            set
            {
                showImage = value;
                pnlPB.Visible = value;
                if (value == false)
                    mainPanel.Padding = new Padding(5 + _level * 8, 0, 0, 0);
                else
                    mainPanel.Padding = new Padding(_level * 8, 0, 0, 0);
            }
        }

        public StackedNodeCollection Nodes
        {
            get
            {
                if (_nodes == null)
                {
                    _nodes = new StackedNodeCollection();
                    _nodes.NodeAdded += OnNodeAdded;

                }
                return _nodes;
            }
        }

        public StackTreeControl StackedTree
        { get; internal set; }

        public StackNodeControl ParentNode
        { get; internal set; }

        public string Key { get; set; }
        public string Status
        {
            get { return lblStatus.Text; }
            set { lblStatus.Text = value; }
        }
        public int Level
        {
            get { return this._level; }
            internal set
            {
                this._level = value;
                this.mainPanel.Padding = new Padding(_level * 8, 0, 0, 0);
            }
        }
        public bool IsSearchable { get; set; } = false;
        public string SearchString { get; set; } = null;

        public bool IsExpanded
        {
            get
            {
                return this.isExpanded;
            }
            set
            {
                if (isExpanded != value)
                {
                    this.isExpanded = value;
                    PerformExpandCollapse();
                }
            }
        }

        public StackNodeControl()
        {
            InitializeComponent();
            this.MouseEnter += StackNodeControl_MouseEnter;
            lblText.MouseEnter += StackNodeControl_MouseEnter;
            mainPanel.MouseEnter += StackNodeControl_MouseEnter;
            pbMain.MouseEnter += StackNodeControl_MouseEnter;

            this.MouseLeave += StackNodeControl_MouseLeave;
            lblText.MouseLeave += StackNodeControl_MouseLeave;
            mainPanel.MouseLeave += StackNodeControl_MouseLeave;
            pbMain.MouseLeave += StackNodeControl_MouseLeave;

            lblText.Click += (s, e) => this.OnNodeClicked(this);
            mainPanel.Click += (s, e) => this.OnNodeClicked(this);
            pbMain.Click += (s, e) => this.OnNodeClicked(this);
            this.Height = 40;
            this.IsVisible = false;
        }

        public void OnNodeClicked(StackNodeControl node)
        {
            StateChangeRequest(null, null);
            if (this.StackNodeClick != null)
            {
                this.StackNodeClick(this.StackedTree, new StackNodeEventArgs(node));
            }
        }

        private void PerformExpandCollapse()
        {
            StackedTree.SuspendDrawing();
            pbMain.Image = this.isExpanded ? ExpandedImage : CollapsedImage;
            pbMain.Refresh();

            foreach (StackNodeControl child in this.Nodes)
                child.IsVisible = isExpanded;
            UpdateHeight();
            StackedTree.ResumeDrawing();
        }

        internal void UpdateHeight()
        {
            this.Height = GetHeight(this);
            var pNode = this.ParentNode;
            while (pNode != null)
            {
                pNode.UpdateHeight();
                pNode = pNode.ParentNode;
            }
        }

        private int GetHeight(StackNodeControl node)
        {
            if (!node.IsVisible || !node.ShouldBeSeen)
                return 0;
            int totalHeight = 40;
            if (!node.IsExpanded)
                return totalHeight;
            else
            {//Get ChildPanel Height
                foreach (StackNodeControl child in node.Nodes)
                {
                    totalHeight += GetHeight(child);
                }
                return totalHeight;
            }
        }


        public bool Has(string Key)
        {
            return Nodes.Any(c => c.Key == Key);
        }

        public void Expand(bool expandAll = false)
        {
            if (expandAll)
            {
                foreach (StackNodeControl child in this.Nodes)
                    child.Expand(expandAll);
            }
            this.IsExpanded = true;
        }

        public void Collapse(bool collapseAll = false)
        {
            if (collapseAll)
            {
                foreach (StackNodeControl child in this.Nodes)
                    child.Collapse(collapseAll);
            }
            this.IsExpanded = false;
        }

        public StackNodeControl FindNode(string key, bool searchAllChild = true)
        {
            StackNodeControl n = null;
            foreach (StackNodeControl node in this.Nodes)
            {
                if (node.Key == key)
                    n = node;
                else
                    n = node.FindNode(key);
                if (n == null)
                    continue;
                else
                    break;
            }
            return n;
        }

        public override string ToString()
        {
            return lblText.Text;
        }

        protected override void OnTextChanged(EventArgs e)
        {
            base.OnTextChanged(e);
            this.lblText.Text = this.Text;
        }

        private void OnNodeAdded(object sender, StackNodeEventArgs e)
        {
            var node = e.Node;
            node.HoverBackColor = this.HoverBackColor;
            node.HoverForeColor = this.HoverForeColor;
            node.ForeColor = this.ForeColor;
            node.BackColor = this.BackColor;
            node.Level = this.Level + 1;
            node.StackedTree = this.StackedTree;
            node.Width = this.Width - mainPanel.Padding.Left;
            node.ParentNode = this;
            node.StackNodeClick += (s, ev) => this.OnNodeClicked(ev);
            this.childPanel.Controls.Add(node);
        }

        private void OnNodeClicked(StackNodeEventArgs e)
        {
            if (StackNodeClick != null)
            {
                StackNodeClick(this, e);
            }
        }

        private void StackNodeControl_MouseLeave(object sender, EventArgs e)
        {
            this.Cursor = Cursors.Default;
        }

        private void StackNodeControl_MouseEnter(object sender, EventArgs e)
        {
            this.Cursor = Cursors.Hand;
        }

        private void StateChangeRequest(object sender, EventArgs e)
        {
            this.IsExpanded = !this.IsExpanded;
        }

        private void OnNodeVisibleChange()
        {
            this.Visible = ShouldBeSeen && IsVisible;
            UpdateHeight();
        }
    }

    public class StackNodeEventArgs : EventArgs
    {
        public StackNodeControl Node { get; private set; }

        public StackNodeEventArgs(StackNodeControl node)
        {
            this.Node = node;
        }
    }
}
