﻿using Spread.Common.Features.Sample;
using Spread.Common.Features.Utilities;
using Spread.ControlExplorer.Components;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;



namespace Spread.Common.Features
{
    public class Loader
    {
        private static Manager Manager;
        private static BaseSample Current;
        private static TileControl tileControl;
        private static SampleHost host;
        private static Sidebar sideBar;
        private static readonly object _locker = new object();

        public static void LoadConfig(string path, Sidebar sidebar, Header header, TileControl tile,SampleHost sampleHost)
        {
            Manager = new Manager(path);
            tileControl = tile;
            host = sampleHost;
            sideBar = sidebar;
            sideBar.DataSource = Manager.Samples;
            header.Title = Manager.Header;
            header.DownloadLink = Manager.DownloadLink;
            header.OnHeaderNavigate += (s, e) =>
            {
                BaseSample sampleToNavigate = null;
                switch(e.Direction)
                {
                    case NavigationDirectionEnum.First:
                        sampleToNavigate = Manager.Samples.First();
                        break;
                    case NavigationDirectionEnum.Last:
                        sampleToNavigate = Manager.Samples.Last();
                        break;
                    case NavigationDirectionEnum.Next:
                        sampleToNavigate = Manager.Samples.GetNext(Current);
                        break;
                    case NavigationDirectionEnum.Previous:
                        sampleToNavigate = Manager.Samples.GetPrevious(Current);
                        break;
                }
                sideBar.SetSelectedSample(sampleToNavigate);
                Current = sampleToNavigate;
            };
        }

        public static Type GetSampleType(BaseSample sample)
        {
            Assembly assembly = null;
            if (!string.IsNullOrWhiteSpace(sample.ExeName))
            {
                try
                {
                    assembly = Assembly.LoadFrom(sample.ExeName);
                }
                catch
                {
                    assembly = Assembly.GetExecutingAssembly();
                }
                
            }
            else
            {
                assembly = Assembly.GetEntryAssembly();
            }
            var types = assembly.GetTypes();
            return types.Where(type => type.IsSubclassOf(typeof(BaseSampleControl))
                        && type.Name.Equals(sample.ClassName))
                        .FirstOrDefault();
        }

        public static void ShowSample(BaseSample sample)
        {
            var t = GetSampleType(sample);
            if (t != null)
            {
                var obj = Activator.CreateInstance(t) as BaseSampleControl;
                if (obj != null)
                {
                    host.Sample = obj;
                    host.Description = sample.Description;
                    host.Code = sample.Code;
                    Current = sample;
                    obj.Dock = DockStyle.Fill;                    
                    host.Visible = true;
                    tileControl.Visible = false;
                }
            }
        }

        public static void ShowTiles(BaseSample baseSample)
        {
            tileControl.Groups.Clear();
            TileGroup group = new TileGroup(baseSample.Title);
            List<BaseSample> samples = new List<BaseSample>();
            if (baseSample is ExampleSample)
                samples.AddRange(Manager.Samples.OfType<ExampleSample>());
            else
                samples.AddRange(Manager.Samples.
                                    OfType<FeatureSample>().
                                    Where(s => s.FeatureName.StartsWith(baseSample.Title)));

            var imagesDir = Path.Combine(Directory.GetCurrentDirectory(), "Images");
            foreach (var sample in samples)
            {
                Image img = null;
                try
                {
                    var exe = sample.ExeName.Replace(".exe", string.Empty);
                    var imageName = string.Format("{0}.{1}.gif",exe,sample.ClassName);
                    var fileName = Path.Combine(imagesDir,imageName);
                    img = Image.FromFile(fileName);
                }
                catch(Exception)
                {
                    img = Properties.Resources.Spread_WaterMark;
                    img.Tag = "PlaceHolder";
                }
                var tile = new TileView
                {
                    Text = sample.Title,
                    Image = img,
                    Tag = sample
                };
                tile.Click += (s, e) => sideBar.SetSelectedSample(sample);
                group.Tiles.Add(tile);
            }
            tileControl.Groups.Add(group);
            host.Visible = false;
            tileControl.Visible = true;
            lock(_locker){
                UpdateImages();
            }
        }

        private static async Task UpdateImages()
        {
            foreach(var group in tileControl.Groups)
            {
                foreach (var tile in group.Tiles.Where(t=>t.Image.Tag!=null))
                {
                   tile.Image = await GetImage(tile.Tag as BaseSample);
                }
            }
        }

        private static Dictionary<string, Image> _imgCache = new Dictionary<string, Image>();

        private static async Task<Image> GetImage(BaseSample sample)
        {
            Image image = null;
            await Task.Factory.StartNew(() => 
            {
            try
            {
                image = _imgCache.FirstOrDefault(img => img.Key == sample.Title).Value;
                if(image==null)
                {
                    var type = GetSampleType(sample);
                    if (type != null)
                    {
                        var obj = Activator.CreateInstance(type) as BaseSampleControl;
                        image = obj.GetImage();
                    }
                }
            }
            catch(Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
             }
                finally
            {
                if(image == null)
                    image = Properties.Resources.Spread_WaterMark;

                if (!_imgCache.ContainsKey(sample.Title))
                    _imgCache.Add(sample.Title, image);
            }
            });
            return image;
        }
    }
}
